@@ -6,6 +6,7 @@ import sys
 import threading
 import time
 from datetime import datetime, timezone, timedelta
+from importlib.metadata import version
 from typing import List, Optional, Tuple, Union, Dict, Any, Callable
 
 import requests
@@ -16,8 +17,6 @@ from eth_account.signers.local import LocalAccount
 from web3 import Web3
 from web3.middleware import ExtraDataToPOAMiddleware
 
-from importlib.metadata import version
-
 from virtuals_acp.configs import ACPContractConfig, DEFAULT_CONFIG
 from virtuals_acp.contract_manager import _ACPContractManager
 from virtuals_acp.exceptions import ACPApiError, ACPError
@@ -49,6 +48,8 @@ class VirtualsACP:
         if not self.w3.is_connected():
             raise ConnectionError(f"Failed to connect to RPC URL: {config.rpc_url}")
 
+        wallet_private_key = wallet_private_key.removeprefix("0x")
+
         self.signer_account: LocalAccount = Account.from_key(wallet_private_key)
 
         if agent_wallet_address:
@@ -109,7 +110,7 @@ class VirtualsACP:
             next_phase=ACPJobPhase(int(memo.get("nextPhase"))),
             status=ACPMemoStatus(memo.get("status")),
             signed_reason=memo.get("signedReason"),
-            expiry=datetime.fromtimestamp(int(memo["expiry"]) / 1000) if memo.get("expiry") else None
+            expiry=datetime.fromtimestamp(int(memo["expiry"])) if memo.get("expiry") else None
         ) for memo in data["memos"]]
 
         memo_to_sign = next(
@@ -147,7 +148,7 @@ class VirtualsACP:
             next_phase=ACPJobPhase(int(memo.get("nextPhase"))),
             status=ACPMemoStatus(memo.get("status")),
             signed_reason=memo.get("signedReason"),
-            expiry=datetime.fromtimestamp(int(memo["expiry"]) / 1000) if memo.get("expiry") else None
+            expiry=datetime.fromtimestamp(int(memo["expiry"])) if memo.get("expiry") else None
         ) for memo in data["memos"]]
 
         context = data["context"]
@@ -259,6 +260,7 @@ class VirtualsACP:
                         provider_address=agent_data["walletAddress"],
                         name=offering["name"],
                         price=offering["price"],
+                        price_usd=offering["priceUsd"],
                         requirement_schema=offering.get("requirementSchema", None)
                     )
                     for offering in agent_data.get("offerings", [])
@@ -293,13 +295,14 @@ class VirtualsACP:
 
         eval_addr = Web3.to_checksum_address(evaluator_address) if evaluator_address else self.agent_address
 
+        if provider_address == self.agent_address:
+            raise Exception("You cannot initiate a job with yourself as the provider")
+
         job_id = None
         retry_count = 3
         retry_delay = 3
 
-        user_op_hash = self.contract_manager.create_job(
-            self.agent_address, provider_address, eval_addr, expired_at
-        )
+        user_op_hash = self.contract_manager.create_job(provider_address, eval_addr, expired_at)
 
         time.sleep(retry_delay)
         for attempt in range(retry_count):
@@ -337,7 +340,7 @@ class VirtualsACP:
                 #     job_id = int(data.get("result").get("jobId"))
 
                 # if job_id is not None and job_id != "":
-                #     break  
+                #     break
 
             except Exception as e:
                 if (attempt == retry_count - 1):
@@ -350,8 +353,7 @@ class VirtualsACP:
         if job_id is None or job_id == "":
             raise Exception("Failed to create job")
 
-        amount_in_wei = self.w3.to_wei(amount, "ether")
-        self.contract_manager.set_budget(job_id, amount_in_wei)
+        self.contract_manager.set_budget_with_payment_token(job_id, amount)
         time.sleep(10)
 
         self.contract_manager.create_memo(
@@ -397,7 +399,7 @@ class VirtualsACP:
             data = self.contract_manager.sign_memo(memo_id, accept, reason or "")
             tx_hash = data.get('receipts', [])[0].get('transactionHash')
             if not accept:
-                exit()
+                return tx_hash
 
             time.sleep(10)
 
@@ -423,10 +425,7 @@ class VirtualsACP:
             reason: Optional[str] = ""
     ) -> Dict[str, Any]:
 
-        amount_in_wei = self.w3.to_wei(amount, "ether")
-        time.sleep(10)
-
-        self.contract_manager.approve_allowance(amount_in_wei)
+        self.contract_manager.approve_allowance(amount)
         time.sleep(10)
 
         self.contract_manager.sign_memo(memo_id, True, reason or "")
@@ -452,18 +451,20 @@ class VirtualsACP:
             fee_type: FeeType,
             reason: GenericPayload[T],
             next_phase: ACPJobPhase,
+            expired_at: datetime
     ) -> str:
         receiver_address = Web3.to_checksum_address(receiver_address)
 
         data = self.contract_manager.create_payable_memo(
             job_id,
             json.dumps(reason.model_dump()),
-            self.w3.to_wei(amount, "ether"),
+            amount,
             receiver_address,
-            self.w3.to_wei(fee_amount, "ether"),
+            fee_amount,
             fee_type,
             next_phase,
-            MemoType.PAYABLE_REQUEST
+            MemoType.PAYABLE_REQUEST,
+            expired_at
         )
 
         tx_hash = data.get('receipts', [])[0].get('transactionHash')
@@ -482,8 +483,7 @@ class VirtualsACP:
             return tx_hash
 
         if amount > 0:
-            amount_in_wei = self.w3.to_wei(amount, "ether")
-            self.contract_manager.approve_allowance(amount_in_wei)
+            self.contract_manager.approve_allowance(amount)
 
         data = self.contract_manager.sign_memo(memo_id, True, reason)
         tx_hash = data.get('receipts', [])[0].get('transactionHash')
@@ -498,24 +498,22 @@ class VirtualsACP:
             fee_type: FeeType,
             reason: GenericPayload[T],
             next_phase: ACPJobPhase,
-            expired_at: Optional[datetime] = None,
+            expired_at: datetime,
     ) -> str:
-        amount_in_wei = self.w3.to_wei(amount, "ether")
-        fee_amount_in_wei = self.w3.to_wei(fee_amount, "ether")
-        total_allowance_in_wei = amount_in_wei + fee_amount_in_wei
+        total_amount = amount + fee_amount
 
-        if (amount + fee_amount) > 0:
-            self.contract_manager.approve_allowance(total_allowance_in_wei)
+        if total_amount > 0:
+            self.contract_manager.approve_allowance(total_amount)
 
         data = self.contract_manager.create_payable_memo(
             job_id,
             json.dumps(reason.model_dump()),
-            amount_in_wei,
+            amount,
             receiver_address,
-            fee_amount_in_wei,
+            fee_amount,
             fee_type,
             next_phase,
-            MemoType.PAYABLE_TRANSFER,
+            MemoType.PAYABLE_TRANSFER_ESCROW,
             expired_at
         )
         tx_hash = data.get('receipts', [])[0].get('transactionHash')
@@ -600,7 +598,7 @@ class VirtualsACP:
                         next_phase=ACPJobPhase(int(memo.get("nextPhase"))),
                         status=ACPMemoStatus(memo.get("status")),
                         signed_reason=memo.get("signedReason"),
-                        expiry=datetime.fromtimestamp(int(memo["expiry"]) / 1000) if memo.get("expiry") else None
+                        expiry=datetime.fromtimestamp(int(memo["expiry"])) if memo.get("expiry") else None
                     ))
 
                 context = job.get("context")
@@ -647,7 +645,7 @@ class VirtualsACP:
                         next_phase=ACPJobPhase(int(memo.get("nextPhase"))),
                         status=ACPMemoStatus(memo.get("status")),
                         signed_reason=memo.get("signedReason"),
-                        expiry=datetime.fromtimestamp(int(memo["expiry"]) / 1000) if memo.get("expiry") else None
+                        expiry=datetime.fromtimestamp(int(memo["expiry"])) if memo.get("expiry") else None
                     ))
 
                 context = job.get("context")
@@ -673,7 +671,7 @@ class VirtualsACP:
             raise ACPApiError(f"Failed to get completed jobs: {e}")
 
     def get_cancelled_jobs(self, page: int = 1, pageSize: int = 10) -> List["ACPJob"]:
-        url = f"{self.acp_api_url}/jobs/cancelled?pagination[page]=${page}&pagination[pageSize]=${pageSize}"
+        url = f"{self.acp_api_url}/jobs/cancelled?pagination[page]={page}&pagination[pageSize]={pageSize}"
         headers = {
             "wallet-address": self.agent_address
         }
@@ -694,7 +692,7 @@ class VirtualsACP:
                         next_phase=ACPJobPhase(int(memo.get("nextPhase"))),
                         status=ACPMemoStatus(memo.get("status")),
                         signed_reason=memo.get("signedReason"),
-                        expiry=datetime.fromtimestamp(int(memo["expiry"]) / 1000) if memo.get("expiry") else None
+                        expiry=datetime.fromtimestamp(int(memo["expiry"])) if memo.get("expiry") else None
                     ))
 
                 context = job.get("context")
@@ -742,7 +740,7 @@ class VirtualsACP:
                     next_phase=ACPJobPhase(int(memo.get("nextPhase"))),
                     status=ACPMemoStatus(memo.get("status")),
                     signed_reason=memo.get("signedReason"),
-                    expiry=datetime.fromtimestamp(int(memo["expiry"]) / 1000) if memo.get("expiry") else None
+                    expiry=datetime.fromtimestamp(int(memo["expiry"])) if memo.get("expiry") else None
                 ))
 
             context = data.get("data", {}).get("context")
@@ -789,7 +787,7 @@ class VirtualsACP:
                 next_phase=ACPJobPhase(int(memo.get("nextPhase"))),
                 status=ACPMemoStatus(memo.get("status")),
                 signed_reason=memo.get("signedReason"),
-                expiry=datetime.fromtimestamp(int(memo["expiry"]) / 1000) if memo.get("expiry") else None
+                expiry=datetime.fromtimestamp(int(memo["expiry"])) if memo.get("expiry") else None
             )
 
         except Exception as e:
@@ -815,6 +813,7 @@ class VirtualsACP:
                     provider_address=agent_data["walletAddress"],
                     name=offering["name"],
                     price=offering["price"],
+                    price_usd=offering["priceUsd"],
                     requirement_schema=offering.get("requirementSchema", None)
                 )
                 for offering in agent_data.get("offerings", [])
