@@ -13,20 +13,11 @@ class EnvSettings(BaseSettings):
     BUYER_ENTITY_ID: Optional[int] = None
     SELLER_ENTITY_ID: Optional[int] = None
     EVALUATOR_ENTITY_ID: Optional[int] = None
-    
-    @field_validator("WHITELISTED_WALLET_PRIVATE_KEY")
-    @classmethod
-    def strip_0x_prefix(cls, v: str) -> str:
-        if v and v.startswith("0x"):
-            raise ValueError("WHITELISTED_WALLET_PRIVATE_KEY must not start with '0x'. Please remove it.")
-        return v
 
     @field_validator("BUYER_AGENT_WALLET_ADDRESS", "SELLER_AGENT_WALLET_ADDRESS", "EVALUATOR_AGENT_WALLET_ADDRESS")
-    @classmethod
     def validate_wallet_address(cls, v: str) -> str:
         if v is None:
             return None
         if not v.startswith("0x") or len(v) != 42:
             raise ValueError("Wallet address must start with '0x' and be 42 characters long.")
         return v
-    
\ No newline at end of file
